import re
from openpyxl.styles.named_styles import NamedStyle
from typing import (
    List,
    Optional,
    Tuple,
    Union,
)

BUILTIN_FORMATS = {
    0: "General",
    1: "0",
    2: "0.00",
    3: "#,##0",
    4: "#,##0.00",
    5: '"$"#,##0_);("$"#,##0)',
    6: '"$"#,##0_);[Red]("$"#,##0)',
    7: '"$"#,##0.00_);("$"#,##0.00)',
    8: '"$"#,##0.00_);[Red]("$"#,##0.00)',
    9: "0%",
    10: "0.00%",
    11: "0.00E+00",
    12: "# ?/?",
    13: "# ??/??",
    14: "mm-dd-yy",
    15: "d-mmm-yy",
    16: "d-mmm",
    17: "mmm-yy",
    18: "h:mm AM/PM",
    19: "h:mm:ss AM/PM",
    20: "h:mm",
    21: "h:mm:ss",
    22: "m/d/yy h:mm",
    37: "#,##0_);(#,##0)",
    38: "#,##0_);[Red](#,##0)",
    39: "#,##0.00_);(#,##0.00)",
    40: "#,##0.00_);[Red](#,##0.00)",
    41: r'_(* #,##0_);_(* \(#,##0\);_(* "-"_);_(@_)',
    42: r'_("$"* #,##0_);_("$"* \(#,##0\);_("$"* "-"_);_(@_)',
    43: r'_(* #,##0.00_);_(* \(#,##0.00\);_(* "-"??_);_(@_)',
    44: r'_("$"* #,##0.00_)_("$"* \(#,##0.00\)_("$"* "-"??_)_(@_)',
    45: "mm:ss",
    46: "[h]:mm:ss",
    47: "mmss.0",
    48: "##0.0E+0",
    49: "@",
}

BUILTIN_FORMATS_MAX_SIZE = 164
BUILTIN_FORMATS_REVERSE = dict([(value, key) for key, value in BUILTIN_FORMATS.items()])

FORMAT_GENERAL = BUILTIN_FORMATS[0]
FORMAT_TEXT = BUILTIN_FORMATS[49]
FORMAT_NUMBER = BUILTIN_FORMATS[1]
FORMAT_NUMBER_00 = BUILTIN_FORMATS[2]
FORMAT_NUMBER_COMMA_SEPARATED1 = BUILTIN_FORMATS[4]
FORMAT_NUMBER_COMMA_SEPARATED2 = "#,##0.00_-"
FORMAT_PERCENTAGE = BUILTIN_FORMATS[9]
FORMAT_PERCENTAGE_00 = BUILTIN_FORMATS[10]
FORMAT_DATE_YYYYMMDD2 = "yyyy-mm-dd"
FORMAT_DATE_YYMMDD = "yy-mm-dd"
FORMAT_DATE_DDMMYY = "dd/mm/yy"
FORMAT_DATE_DMYSLASH = "d/m/y"
FORMAT_DATE_DMYMINUS = "d-m-y"
FORMAT_DATE_DMMINUS = "d-m"
FORMAT_DATE_MYMINUS = "m-y"
FORMAT_DATE_XLSX14 = BUILTIN_FORMATS[14]
FORMAT_DATE_XLSX15 = BUILTIN_FORMATS[15]
FORMAT_DATE_XLSX16 = BUILTIN_FORMATS[16]
FORMAT_DATE_XLSX17 = BUILTIN_FORMATS[17]
FORMAT_DATE_XLSX22 = BUILTIN_FORMATS[22]
FORMAT_DATE_DATETIME = "yyyy-mm-dd h:mm:ss"
FORMAT_DATE_TIME1 = BUILTIN_FORMATS[18]
FORMAT_DATE_TIME2 = BUILTIN_FORMATS[19]
FORMAT_DATE_TIME3 = BUILTIN_FORMATS[20]
FORMAT_DATE_TIME4 = BUILTIN_FORMATS[21]
FORMAT_DATE_TIME5 = BUILTIN_FORMATS[45]
FORMAT_DATE_TIME6 = BUILTIN_FORMATS[21]
FORMAT_DATE_TIME7 = "i:s.S"
FORMAT_DATE_TIME8 = "h:mm:ss@"
FORMAT_DATE_TIMEDELTA = "[hh]:mm:ss"
FORMAT_DATE_YYMMDDSLASH = "yy/mm/dd@"
FORMAT_CURRENCY_USD_SIMPLE = '"$"#,##0.00_-'
FORMAT_CURRENCY_USD = "$#,##0_-"
FORMAT_CURRENCY_EUR_SIMPLE = "[$EUR ]#,##0.00_-"

COLORS = r"\[(BLACK|BLUE|CYAN|GREEN|MAGENTA|RED|WHITE|YELLOW)\]"
LITERAL_GROUP = r'".*?"'  # anything in quotes
LOCALE_GROUP = r"\[.+\]"  # anything in square brackets, including colours
STRIP_RE = re.compile(f"{LITERAL_GROUP}|{LOCALE_GROUP}")

def builtin_format_code(index: int) -> str: ...
def is_date_format(fmt: str) -> bool: ...
def is_datetime(fmt: str) -> str: ...

class NumberFormat:
    def __init__(
        self, numFmtId: Optional[Union[str, int]] = ..., formatCode: Optional[str] = ...
    ) -> None: ...

class NumberFormatDescriptor:
    def __set__(self, instance: NamedStyle, value: Optional[str]) -> None: ...

class NumberFormatList:
    def __init__(
        self, count: Optional[str] = ..., numFmt: Union[List[NumberFormat], Tuple] = ...
    ) -> None: ...
    @property
    def count(self) -> int: ...

def is_builtin(fmt):
    return fmt in BUILTIN_FORMATS.values()



from lxml.etree import _Element
from openpyxl.styles.colors import Color
from typing import (
    Iterator,
    List,
    Optional,
    Tuple,
    Union,
)

def _assign_position(
    values: Union[List[Color], List[Stop], Tuple, List[Union[Stop, str]], List[str]]
) -> List[Stop]: ...

class Fill:
    @classmethod
    def from_tree(cls, el: _Element) -> Optional[Union[GradientFill, PatternFill]]: ...

class GradientFill:
    def __init__(
        self,
        type: str = ...,
        degree: Union[str, int] = ...,
        left: Union[str, int] = ...,
        right: Union[str, int] = ...,
        top: Union[str, int] = ...,
        bottom: Union[str, int] = ...,
        stop: Union[List[Stop], Tuple] = ...,
    ) -> None: ...
    def __iter__(self) -> Iterator[Tuple[str, str]]: ...
    def to_tree(
        self, tagname: None = ..., namespace: None = ..., idx: None = ...
    ) -> _Element: ...

class PatternFill:
    def __init__(
        self,
        patternType: Optional[str] = ...,
        fgColor: Union[str, Color] = ...,
        bgColor: Union[str, Color] = ...,
        fill_type: Optional[str] = ...,
        start_color: Optional[Union[str, Color]] = ...,
        end_color: Optional[Union[str, Color]] = ...,
    ) -> None: ...
    @classmethod
    def _from_tree(cls, el: _Element) -> PatternFill: ...
    def to_tree(self, tagname: Optional[str] = ..., idx: None = ...) -> _Element: ...

class Stop:
    def __init__(
        self, color: Union[str, Color], position: Optional[Union[float, str, int]]
    ) -> None: ...

class StopList:
    def __set__(self, obj: GradientFill, values: Union[List[Stop], Tuple]) -> None: ...



from lxml.etree import _Element
from openpyxl.chart.data_source import NumFmt
from openpyxl.chart.shapes import GraphicalProperties
from openpyxl.chart.text import RichText
from openpyxl.chart.title import Title
from typing import (
    Optional,
    Union,
)

class ChartLines:
    def __init__(self, spPr: Optional[GraphicalProperties] = ...) -> None: ...

class DateAxis:
    def __init__(
        self,
        auto: Optional[Union[str, bool]] = ...,
        lblOffset: Optional[Union[str, int]] = ...,
        baseTimeUnit: Optional[str] = ...,
        majorUnit: Optional[str] = ...,
        majorTimeUnit: Optional[str] = ...,
        minorUnit: None = ...,
        minorTimeUnit: None = ...,
        extLst: None = ...,
        **kw
    ) -> None: ...

class DisplayUnitsLabel:
    def __init__(
        self, layout: None = ..., tx: None = ..., spPr: None = ..., txPr: None = ...
    ) -> None: ...

class DisplayUnitsLabelList:
    def __init__(
        self,
        custUnit: None = ...,
        builtInUnit: Optional[str] = ...,
        dispUnitsLbl: None = ...,
        extLst: None = ...,
    ) -> None: ...

class NumericAxis:
    def __init__(
        self,
        crossBetween: Optional[str] = ...,
        majorUnit: Optional[str] = ...,
        minorUnit: Optional[str] = ...,
        dispUnits: Optional[DisplayUnitsLabelList] = ...,
        extLst: None = ...,
        **kw
    ) -> None: ...
    @classmethod
    def from_tree(cls, node: _Element) -> NumericAxis: ...

class Scaling:
    def __init__(
        self,
        logBase: Optional[Union[str, int]] = ...,
        orientation: str = ...,
        max: Optional[str] = ...,
        min: Optional[str] = ...,
        extLst: None = ...,
    ) -> None: ...

class SeriesAxis:
    def __init__(
        self,
        tickLblSkip: None = ...,
        tickMarkSkip: None = ...,
        extLst: None = ...,
        **kw
    ) -> None: ...

class TextAxis:
    def __init__(
        self,
        auto: Optional[str] = ...,
        lblAlgn: Optional[str] = ...,
        lblOffset: Optional[Union[str, int]] = ...,
        tickLblSkip: None = ...,
        tickMarkSkip: None = ...,
        noMultiLvlLbl: Optional[str] = ...,
        extLst: None = ...,
        **kw
    ) -> None: ...

class _BaseAxis:
    def __init__(
        self,
        axId: Optional[Union[str, int]] = ...,
        scaling: Optional[Scaling] = ...,
        delete: Optional[Union[str, bool]] = ...,
        axPos: str = ...,
        majorGridlines: Optional[ChartLines] = ...,
        minorGridlines: None = ...,
        title: Optional[Title] = ...,
        numFmt: Optional[NumFmt] = ...,
        majorTickMark: Optional[str] = ...,
        minorTickMark: Optional[str] = ...,
        tickLblPos: Optional[str] = ...,
        spPr: Optional[GraphicalProperties] = ...,
        txPr: Optional[RichText] = ...,
        crossAx: Optional[Union[str, int]] = ...,
        crosses: Optional[str] = ...,
        crossesAt: Optional[str] = ...,
    ) -> None: ...



from openpyxl.packaging.relationship import (
    Relationship,
    RelationshipList,
)
from openpyxl.packaging.workbook import ChildSheet
from openpyxl.pivot.cache import CacheDefinition
from typing import (
    Dict,
    Iterator,
    Tuple,
)
from zipfile import ZipFile

class WorkbookParser:
    def __init__(
        self, archive: ZipFile, workbook_part_name: str, keep_links: bool = ...
    ) -> None: ...
    def assign_names(self) -> None: ...
    def find_sheets(self) -> Iterator[Tuple[ChildSheet, Relationship]]: ...
    def parse(self) -> None: ...
    @property
    def pivot_caches(self) -> Dict[int, CacheDefinition]: ...
    @property
    def rels(self) -> RelationshipList: ...



from openpyxl.styles.colors import Color
from typing import (
    Iterator,
    Optional,
    Tuple,
    Union,
)

class Border:
    def __init__(
        self,
        left: Side = ...,
        right: Side = ...,
        top: Side = ...,
        bottom: Side = ...,
        diagonal: Side = ...,
        diagonal_direction: None = ...,
        vertical: None = ...,
        horizontal: None = ...,
        diagonalUp: Union[str, bool] = ...,
        diagonalDown: Union[str, bool] = ...,
        outline: bool = ...,
        start: None = ...,
        end: None = ...,
    ) -> None: ...
    def __iter__(self) -> Iterator[Tuple[str, str]]: ...

class Side:
    def __init__(
        self,
        style: Optional[str] = ...,
        color: Optional[Union[Color, str]] = ...,
        border_style: Optional[str] = ...,
    ) -> None: ...



from openpyxl.cell.cell import (
    Cell,
    MergedCell,
)
from openpyxl.cell.read_only import ReadOnlyCell
from openpyxl.chart.bar_chart import BarChart
from openpyxl.drawing.image import Image
from openpyxl.workbook.workbook import Workbook
from openpyxl.worksheet.cell_range import CellRange
from openpyxl.worksheet.dimensions import (
    ColumnDimension,
    RowDimension,
)
from openpyxl.worksheet.merge import MergedCellRange
from openpyxl.worksheet.table import (
    Table,
    TableList,
)
from openpyxl.worksheet.tests.test_dimensions import DummyWorkbook
from openpyxl.worksheet.tests.test_worksheet import DummyWorkbook
from openpyxl.worksheet.views import SheetView
from typing import (
    Any,
    Iterator,
    Optional,
    Tuple,
    Union,
)

def _gutter(idx: int, offset: int, max_val: int) -> range: ...

class Worksheet:
    def __delitem__(self, key: str) -> None: ...
    def __getitem__(self, key: Union[str, slice, int]) -> Any: ...
    def __init__(
        self,
        parent: Union[DummyWorkbook, DummyWorkbook, Workbook],
        title: Optional[Union[str, int]] = ...,
    ) -> None: ...
    def __iter__(self) -> Iterator[Any]: ...
    def __setitem__(self, key: str, value: Union[str, int]) -> None: ...
    def _add_cell(self, cell: Cell) -> None: ...
    def _add_column(self) -> ColumnDimension: ...
    def _add_row(self) -> RowDimension: ...
    def _cells_by_col(
        self,
        min_col: int,
        min_row: int,
        max_col: int,
        max_row: int,
        values_only: bool = ...,
    ) -> Iterator[Tuple[Cell, ...]]: ...
    def _cells_by_row(
        self,
        min_col: int,
        min_row: int,
        max_col: int,
        max_row: int,
        values_only: bool = ...,
    ) -> Iterator[Any]: ...
    def _clean_merge_range(self, mcr: MergedCellRange) -> None: ...
    def _get_cell(self, row: int, column: int) -> Union[MergedCell, Cell]: ...
    def _invalid_row(self, iterable: str): ...
    def _move_cell(
        self,
        row: int,
        column: int,
        row_offset: int,
        col_offset: int,
        translate: bool = ...,
    ) -> None: ...
    def _move_cells(
        self,
        min_row: Optional[int] = ...,
        min_col: Optional[int] = ...,
        offset: int = ...,
        row_or_col: str = ...,
    ) -> None: ...
    def _setup(self) -> None: ...
    @property
    def active_cell(self) -> str: ...
    def add_chart(self, chart: BarChart, anchor: Optional[str] = ...) -> None: ...
    def add_image(self, img: Image, anchor: Optional[str] = ...) -> None: ...
    def add_table(self, table: Table) -> None: ...
    def append(self, iterable: Any) -> None: ...
    def calculate_dimension(self) -> str: ...
    def cell(
        self, row: int, column: int, value: Optional[int] = ...
    ) -> Union[MergedCell, ReadOnlyCell, Cell]: ...
    @property
    def columns(self) -> Iterator[Any]: ...
    def delete_cols(self, idx: int, amount: int = ...) -> None: ...
    def delete_rows(self, idx: int, amount: int = ...) -> None: ...
    def insert_cols(self, idx: int, amount: int = ...) -> None: ...
    def insert_rows(self, idx: int, amount: int = ...) -> None: ...
    def iter_cols(
        self,
        min_col: Optional[int] = ...,
        max_col: Optional[int] = ...,
        min_row: None = ...,
        max_row: None = ...,
        values_only: bool = ...,
    ) -> Iterator[Any]: ...
    def iter_rows(
        self,
        min_row: Optional[int] = ...,
        max_row: Optional[int] = ...,
        min_col: Optional[int] = ...,
        max_col: Optional[int] = ...,
        values_only: bool = ...,
    ) -> Iterator[Any]: ...
    @property
    def max_column(self) -> int: ...
    @property
    def max_row(self) -> int: ...
    def merge_cells(
        self,
        range_string: Optional[str] = ...,
        start_row: Optional[int] = ...,
        start_column: Optional[int] = ...,
        end_row: Optional[int] = ...,
        end_column: Optional[int] = ...,
    ) -> None: ...
    @property
    def min_column(self) -> int: ...
    @property
    def min_row(self) -> int: ...
    def move_range(
        self,
        cell_range: Union[CellRange, str],
        rows: int = ...,
        cols: int = ...,
        translate: bool = ...,
    ) -> None: ...
    @property
    def print_titles(self) -> Optional[str]: ...
    @property
    def rows(self) -> Iterator[Any]: ...
    @property
    def selected_cell(self) -> str: ...
    @property
    def sheet_view(self) -> SheetView: ...
    @property
    def show_gridlines(self) -> None: ...
    @property
    def tables(self) -> TableList: ...
    def unmerge_cells(
        self,
        range_string: Optional[str] = ...,
        start_row: Optional[int] = ...,
        start_column: Optional[int] = ...,
        end_row: Optional[int] = ...,
        end_column: Optional[int] = ...,
    ) -> None: ...
    @property
    def values(
        self,
    ) -> Iterator[
        Union[
            Tuple[str, None, None, None, None, None, None],
            Tuple[None, None, None, None, None, None, None],
            Tuple[None, None, None, None, None, None, str],
        ]
    ]: ...



import datetime
import re
from openpyxl.styles.cell_style import StyleArray
from openpyxl.worksheet._write_only import WriteOnlyWorksheet
from openpyxl.worksheet.worksheet import Worksheet
from openpyxl.styles import numbers
from openpyxl.styles.styleable import StyleableObject
from typing import Any, Optional, Union, Callable, Tuple, Dict
from openpyxl.compat import NUMERIC_TYPES

TIME_TYPES = (datetime.datetime, datetime.date, datetime.time, datetime.timedelta)
TIME_FORMATS = {
    datetime.datetime: numbers.FORMAT_DATE_DATETIME,
    datetime.date: numbers.FORMAT_DATE_YYYYMMDD2,
    datetime.time: numbers.FORMAT_DATE_TIME6,
    datetime.timedelta: numbers.FORMAT_DATE_TIMEDELTA,
}

STRING_TYPES = (str, bytes)
KNOWN_TYPES = NUMERIC_TYPES + TIME_TYPES + STRING_TYPES + (bool, type(None))

ILLEGAL_CHARACTERS_RE = re.compile(r"[\000-\010]|[\013-\014]|[\016-\037]")
ERROR_CODES = ("#NULL!", "#DIV/0!", "#VALUE!", "#REF!", "#NAME?", "#NUM!", "#N/A")

ERROR_CODES = ERROR_CODES

TYPE_STRING: str = "s"
TYPE_FORMULA: str = "f"
TYPE_NUMERIC: str = "n"
TYPE_BOOL: str = "b"
TYPE_NULL: str = "n"
TYPE_INLINE: str = "inlineStr"
TYPE_ERROR: str = "e"
TYPE_FORMULA_CACHE_STRING: str = "str"

VALID_TYPES: Tuple[str, ...] = (
    TYPE_STRING,
    TYPE_FORMULA,
    TYPE_NUMERIC,
    TYPE_BOOL,
    TYPE_NULL,
    TYPE_INLINE,
    TYPE_ERROR,
    TYPE_FORMULA_CACHE_STRING,
)

_TYPES: Dict[Callable, str] = {int: "n", float: "n", str: "s", bool: "b"}

def WriteOnlyCell(
    ws: Optional[WriteOnlyWorksheet] = ..., value: Optional[Union[str, int]] = ...
) -> Cell: ...
def get_time_format(t: Any) -> Optional[str]: ...
def get_type(t: Any, value: Any) -> Optional[str]: ...

class Cell(StyleableObject):
    def __init__(
        self,
        worksheet: Optional[Union[Worksheet, WriteOnlyWorksheet]],
        row: Optional[Union[str, int]] = ...,
        column: Optional[int] = ...,
        value: Optional[Union[datetime.date, str, int]] = ...,
        style_array: Optional[StyleArray] = ...,
    ) -> None: ...
    def __init__(self, worksheet, row=None, column=None, value=None, style_array=None):
        super(Cell, self).__init__(worksheet, style_array)
        self.row = row
        """Row number of this cell (1-based)"""
        self.column = column
        """Column number of this cell (1-based)"""
        # _value is the stored value, while value is the displayed value
        self._value = None
        self._hyperlink = None
        self.data_type = "n"
        if value is not None:
            self.value = value
        self._comment = None
    def __repr__(self) -> str: ...
    def _bind_value(self, value: Any) -> None: ...
    def check_string(self, value: Union[str, bytes]) -> str: ...
    @property
    def col_idx(self) -> int: ...
    @property
    def coordinate(self) -> str: ...
    @property
    def encoding(self) -> str: ...
    @property
    def is_date(self) -> bool: ...
    def offset(self, row: int = ..., column: int = ...) -> Cell: ...

class MergedCell:
    def __init__(
        self,
        worksheet: Worksheet,
        row: Optional[int] = ...,
        column: Optional[int] = ...,
    ) -> None: ...
    def __repr__(self) -> str: ...



from openpyxl.styles.borders import Border
from openpyxl.styles.fills import PatternFill
from openpyxl.styles.fonts import Font
from typing import (
    Optional,
    Tuple,
)

class DifferentialStyle:
    def __init__(
        self,
        font: Optional[Font] = ...,
        numFmt: None = ...,
        fill: Optional[PatternFill] = ...,
        alignment: None = ...,
        border: Optional[Border] = ...,
        protection: None = ...,
        extLst: None = ...,
    ) -> None: ...

class DifferentialStyleList:
    def __getitem__(self, idx: int) -> DifferentialStyle: ...
    def __init__(self, dxf: Tuple = ...) -> None: ...
    def add(self, dxf: DifferentialStyle) -> int: ...
    def append(self, dxf: DifferentialStyle) -> None: ...



from lxml.etree import _Element
from openpyxl.cell.text import InlineFont
from openpyxl.styles.colors import Color
from typing import (
    Optional,
    Union,
)
from xml.etree.ElementTree import Element

def _no_value(
    tagname: str, value: bool, namespace: None = ...
) -> Optional[_Element]: ...

class Font:
    def __init__(
        self,
        name: Optional[str] = ...,
        sz: Optional[Union[str, int]] = ...,
        b: Optional[Union[str, bool]] = ...,
        i: Optional[Union[str, bool]] = ...,
        charset: Optional[Union[str, int]] = ...,
        u: Optional[str] = ...,
        strike: Optional[bool] = ...,
        color: Optional[Union[Color, str]] = ...,
        scheme: Optional[str] = ...,
        family: Optional[Union[str, int]] = ...,
        size: Optional[int] = ...,
        bold: Optional[bool] = ...,
        italic: Optional[bool] = ...,
        strikethrough: Optional[bool] = ...,
        underline: Optional[str] = ...,
        vertAlign: Optional[str] = ...,
        outline: None = ...,
        shadow: None = ...,
        condense: Optional[str] = ...,
        extend: Optional[str] = ...,
    ) -> None: ...
    @classmethod
    def from_tree(cls, node: Union[_Element, Element]) -> Union[InlineFont, Font]: ...

DEFAULT_FONT = Font(
    name="Calibri",
    sz=11,
    family=2,
    b=False,
    i=False,
    color=Color(theme=1),
    scheme="minor",
)



from typing import (
    Iterator,
    List,
    Tuple,
    Union,
)

def _get_column_letter(col_idx: int) -> str: ...
def absolute_coordinate(coord_string: str) -> str: ...
def cols_from_range(range_string: str) -> Iterator[Tuple[str, str, str, str]]: ...
def column_index_from_string(str_col: str) -> int: ...
def coordinate_from_string(coord_string: str) -> Tuple[str, int]: ...
def coordinate_to_tuple(coordinate: str) -> Tuple[int, int]: ...
def get_column_interval(start: str, end: Union[str, int]) -> List[str]: ...
def get_column_letter(idx: int) -> str: ...
def quote_sheetname(sheetname: str) -> str: ...
def range_boundaries(
    range_string: str,
) -> Union[
    Tuple[None, int, None, int], Tuple[int, int, int, int], Tuple[int, None, int, None]
]: ...
def range_to_tuple(range_string: str) -> Tuple[str, Tuple[int, int, int, int]]: ...
def rows_from_range(
    range_string: str,
) -> Iterator[Union[Tuple[str], Tuple[str, str, str, str]]]: ...



from openpyxl.chart.line_chart import LineChart
from openpyxl.chart.scatter_chart import ScatterChart
from openpyxl.drawing.image import Image
from typing import (
    Any,
    List,
    Tuple,
    Union,
)
from zipfile import ZipFile

def find_images(
    archive: ZipFile, path: str
) -> Union[
    Tuple[List[ScatterChart], List[Any]],
    Tuple[List[Any], List[Any]],
    Tuple[List[Any], List[Image]],
    Tuple[List[LineChart], List[Any]],
]: ...



from .alignment import Alignment as Alignment
from .borders import Border as Border, Side as Side
from .colors import Color as Color
from .fills import PatternFill as PatternFill, GradientFill as GradientFill, Fill as Fill
from .fonts import Font as Font, DEFAULT_FONT as DEFAULT_FONT
from .numbers import NumberFormatDescriptor as NumberFormatDescriptor, is_date_format as is_date_format, is_builtin as is_builtin
from .protection import Protection as Protection
from .named_styles import NamedStyle as NamedStyle



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

# Core Library modules
import os

# Third party modules
from setuptools import setup


def find_stub_files():
    """
    This function is taken from the awesome sqlalchey-stubs:
    https://github.com/dropbox/sqlalchemy-stubs/blob/master/setup.py#L32
    It's licensed under Apache 2.0:
    https://github.com/dropbox/sqlalchemy-stubs/blob/master/LICENSE
    """
    result = []
    for root, dirs, files in os.walk("openpyxl-stubs"):
        for file in files:
            if file.endswith(".pyi"):
                if os.path.sep in root:
                    sub_root = root.split(os.path.sep, 1)[-1]
                    file = os.path.join(sub_root, file)
                result.append(file)
    return result


setup(
    name="openpyxl-stubs",
    install_requires=[
        "mypy>=0.720",
        "typing-extensions>=3.7.4",
        "openpyxl>=3.0.0",
    ],
    packages=["openpyxl-stubs"],
    package_data={"openpyxl-stubs": find_stub_files()},
)



from typing import (
    Iterator,
    Optional,
    Tuple,
    Union,
)

class Alignment:
    def __init__(
        self,
        horizontal: Optional[str] = ...,
        vertical: Optional[str] = ...,
        textRotation: Union[str, int] = ...,
        wrapText: Optional[Union[str, bool]] = ...,
        shrinkToFit: Optional[str] = ...,
        indent: Union[str, int] = ...,
        relativeIndent: int = ...,
        justifyLastLine: None = ...,
        readingOrder: int = ...,
        text_rotation: Optional[int] = ...,
        wrap_text: Optional[bool] = ...,
        shrink_to_fit: Optional[bool] = ...,
        mergeCell: None = ...,
    ) -> None: ...
    def __iter__(self) -> Iterator[Tuple[str, str]]: ...



from openpyxl.styles.colors import Color
from typing import (
    List,
    Optional,
    Tuple,
    Union,
)

class InlineFont:
    def __init__(
        self,
        rFont: Optional[str] = ...,
        charset: Optional[str] = ...,
        family: Optional[str] = ...,
        b: Optional[bool] = ...,
        i: None = ...,
        strike: None = ...,
        outline: None = ...,
        shadow: None = ...,
        condense: None = ...,
        extend: None = ...,
        color: Optional[Color] = ...,
        sz: Optional[str] = ...,
        u: Optional[str] = ...,
        vertAlign: None = ...,
        scheme: Optional[str] = ...,
    ) -> None: ...

class PhoneticProperties:
    def __init__(
        self,
        fontId: Optional[Union[str, int]] = ...,
        type: Optional[str] = ...,
        alignment: None = ...,
    ) -> None: ...

class PhoneticText:
    def __init__(
        self,
        sb: Optional[Union[str, int]] = ...,
        eb: Optional[Union[str, int]] = ...,
        t: Optional[str] = ...,
    ) -> None: ...

class RichText:
    def __init__(
        self, rPr: Optional[InlineFont] = ..., t: Optional[str] = ...
    ) -> None: ...

class Text:
    def __init__(
        self,
        t: Optional[str] = ...,
        r: Union[Tuple, List[RichText]] = ...,
        rPh: Tuple = ...,
        phoneticPr: None = ...,
    ) -> None: ...
    @property
    def content(self) -> str: ...
